@@ -27,8 +27,6 @@
  *  a) Speed optimisation of passing a raw pointer to the mapped stream back
  *  b) Merging continues, if a record is too large for a single BIFF record
  *     ie. 0xffff bytes ( or more usu. 0x2000 ) it is split into continues.
- *  c) However continues do not always behave as specified requiring an arbitary
- *     number of padding bytes occasionaly ( see ms_bug_get_padding ).
  **/
 
 /*******************************************************************************/
@@ -69,38 +67,16 @@ dump_biff (BiffQuery *bq)
 /*	dump_stream (bq->pos); */
 }
 
-static guint16
-ms_bug_get_padding (guint16 opcode)
-{
-	guint8 ls_op = (opcode & 0x00ff);
-	guint ans=0;
-
-	switch (ls_op)
-	{
-	case BIFF_SST:
-		ans=1;
-		break;
-	case BIFF_MS_O_DRAWING_GROUP:
-	case BIFF_MS_O_DRAWING:
-		ans=0;
-		break;
-	default:
-#if BIFF_DEBUG > 0
-		printf ("Unknown padding to fix bug on record 0x%x\n", opcode);
-#endif
-		break;
-	}
-#if BIFF_DEBUG > 0
-	printf ("ms_bug_get_padding 0x%x = %d\n",
-		opcode, ans);
-#endif
-	return ans;
-}
-
 /*******************************************************************************/
 /*                                 Read Side                                   */
 /*******************************************************************************/
 
+/*static guint16
+no_quirks (const BiffQuery *q, guint16 op)
+{
+	return 0;
+}*/
+
 BiffQuery *
 ms_biff_query_new (MsOleStream *ptr)
 {
@@ -111,15 +87,24 @@ ms_biff_query_new (MsOleStream *ptr)
 	bq->opcode        = 0;
 	bq->length        = 0;
 	bq->data_malloced = 0;
-	bq->padding       = 0;
+/*	bq->padding       = 0; */
 	bq->num_merges    = 0;
 	bq->pos           = ptr;
+/*	bq->quirk         = no_quirks;*/
 #if BIFF_DEBUG > 0
 	dump_biff(bq);
 #endif
 	return bq;
 }
 
+/*void
+ms_biff_query_set_quirk (BiffQuery *bq, BiffQuirkFn *quirk)
+{
+	g_return_if_fail (bq != NULL);
+
+	bq->quirk = quirk;
+}*/
+
 BiffQuery *
 ms_biff_query_copy (const BiffQuery *p)
 {
@@ -163,7 +148,7 @@ ms_biff_merge_continues (BiffQuery *bq, guint32 len)
 	total_len = chunk.length;
 	g_array_append_val (contin, chunk);
 
-	bq->padding = ms_bug_get_padding (bq->opcode);
+/*	bq->padding = bq->quirk (bq, bq->opcode); */
 
 	/* Subsequent continue blocks */
 	chunk.length = len;
@@ -179,12 +164,12 @@ ms_biff_merge_continues (BiffQuery *bq, guint32 len)
 #endif
 		tmp[0] = 0; tmp[1] = 0; tmp[2] = 0; tmp[3] = 0;
 		bq->pos->read_copy (bq->pos, tmp, 4);	
-		total_len   += chunk.length - bq->padding;
+		total_len   += chunk.length/* - bq->padding*/;
 		g_array_append_val (contin, chunk);
 
-		chunk.length = BIFF_GET_GUINT16 (tmp+2);
+		chunk.length = MS_OLE_GET_GUINT16 (tmp+2);
 		bq->num_merges++;
-	} while ((BIFF_GET_GUINT16(tmp) & 0xff) == BIFF_CONTINUE);
+	} while ((MS_OLE_GET_GUINT16(tmp) & 0xff) == BIFF_CONTINUE);
 	bq->pos->lseek (bq->pos, -4, MsOleSeekCur); /* back back off */
 
 	bq->data = g_malloc (total_len);
@@ -201,8 +186,8 @@ ms_biff_merge_continues (BiffQuery *bq, guint32 len)
 		g_assert ((d-bq->data)+chunk.length<=total_len);
 #endif
 		if (lp) {
-			memcpy (d, chunk.data+bq->padding, chunk.length-bq->padding);
-			d+=chunk.length-bq->padding;
+			memcpy (d, chunk.data/*+bq->padding*/, chunk.length/*-bq->padding*/);
+			d+=chunk.length/*-bq->padding*/;
 		} else {
 			memcpy (d, chunk.data, chunk.length);
 			d+=chunk.length;
@@ -218,6 +203,24 @@ ms_biff_merge_continues (BiffQuery *bq, guint32 len)
 	return 1;
 }
 
+int
+ms_biff_query_peek_next (BiffQuery *bq, guint16 *opcode)
+{
+	guint8 data[4];
+	g_return_val_if_fail (opcode != NULL, 0);
+
+	if (!bq || (bq->pos->position + 4 > bq->pos->size))
+		return 0;
+
+	if (!bq->pos->read_copy (bq->pos, data, 4))
+		return 0;
+
+	bq->pos->lseek (bq->pos, -4, MsOleSeekCur); /* back back off */
+
+	*opcode = MS_OLE_GET_GUINT16 (data);
+	return 1;
+}
+
 /**
  * Returns 0 if has hit end
  **/
@@ -232,7 +235,7 @@ ms_biff_query_next_merge (BiffQuery *bq, gboolean do_merge)
 	if (bq->data_malloced) { /* always true for merged records*/
 		g_free (bq->data);
 		bq->num_merges    = 0;
-		bq->padding       = 0;
+/*		bq->padding       = 0;*/
 		bq->data_malloced = 0;
 	}
 	g_assert (bq->num_merges == 0);
@@ -240,8 +243,8 @@ ms_biff_query_next_merge (BiffQuery *bq, gboolean do_merge)
 	bq->streamPos = bq->pos->position;
 	if (!bq->pos->read_copy (bq->pos, tmp, 4))
 		return 0;
-	bq->opcode = BIFF_GET_GUINT16 (tmp);
-	bq->length = BIFF_GET_GUINT16 (tmp+2);
+	bq->opcode = MS_OLE_GET_GUINT16 (tmp);
+	bq->length = MS_OLE_GET_GUINT16 (tmp+2);
 	bq->ms_op  = (bq->opcode>>8);
 	bq->ls_op  = (bq->opcode&0xff);
 
@@ -257,8 +260,8 @@ ms_biff_query_next_merge (BiffQuery *bq, gboolean do_merge)
 	}
 	if (ans && do_merge &&
 	    bq->pos->read_copy (bq->pos, tmp, 4)) {
-		if ((BIFF_GET_GUINT16(tmp) & 0xff) == BIFF_CONTINUE)
-			return ms_biff_merge_continues (bq, BIFF_GET_GUINT16(tmp+2));
+		if ((MS_OLE_GET_GUINT16(tmp) & 0xff) == BIFF_CONTINUE)
+			return ms_biff_merge_continues (bq, MS_OLE_GET_GUINT16(tmp+2));
 		bq->pos->lseek (bq->pos, -4, MsOleSeekCur); /* back back off */
 #if BIFF_DEBUG > 4
 		printf ("Backed off\n");
@@ -282,7 +285,7 @@ ms_biff_query_unmerge (BiffQuery *bq)
 {
 	if (!bq || !bq->num_merges)
 		return;
-	bq->pos->lseek (bq->pos, -(4*(bq->num_merges+1) - (bq->num_merges*bq->padding)
+	bq->pos->lseek (bq->pos, -(4*(bq->num_merges+1)/* - (bq->num_merges*bq->padding)*/
 				   + bq->length), MsOleSeekCur);
 	ms_biff_query_next_merge (bq, FALSE);
 }
@@ -306,10 +309,10 @@ ms_biff_query_destroy (BiffQuery *bq)
 
 /* Sets up a record on a stream */
 BiffPut *
-ms_biff_put_new        (MsOleStream *s)
+ms_biff_put_new (MsOleStream *s)
 {
 	BiffPut *bp;
-	g_return_val_if_fail (s, 0);
+	g_return_val_if_fail (s != NULL, 0);
 
 	bp = g_new (BiffPut, 1);
 
@@ -318,17 +321,19 @@ ms_biff_put_new        (MsOleStream *s)
 	bp->length        = 0;
 	bp->streamPos     = s->tell (s);
 	bp->num_merges    = 0;
-	bp->padding       = 0;
+/*	bp->padding       = 0;*/
 	bp->data_malloced = 0;
 	bp->len_fixed     = 0;
 	bp->pos           = s;
+/*	bp->quirk         = no_quirks;*/
 	return bp;
 }
+
 void
-ms_biff_put_destroy    (BiffPut *bp)
+ms_biff_put_destroy (BiffPut *bp)
 {
-	g_return_if_fail (bp);
-	g_return_if_fail (bp->pos);
+	g_return_if_fail (bp != NULL);
+	g_return_if_fail (bp->pos != NULL);
 
 	g_free (bp);
 }
@@ -344,7 +349,7 @@ ms_biff_put_len_next   (BiffPut *bp, guint16 opcode, guint32 len)
 	bp->ms_op      = (opcode >>   8);
 	bp->ls_op      = (opcode & 0xff);
 	bp->length     = len;
-	bp->padding    = 0;
+/*	bp->padding    = 0;*/
 	bp->num_merges = 0;
 	bp->streamPos  = bp->pos->tell (bp->pos);
 	if (len > 0)
@@ -364,15 +369,15 @@ ms_biff_put_var_next   (BiffPut *bp, guint16 opcode)
 	bp->len_fixed  = 0;
 	bp->ms_op      = (opcode >>   8);
 	bp->ls_op      = (opcode & 0xff);
-	bp->padding    = ms_bug_get_padding (opcode);
+/*	bp->padding    = 0;*/
 	bp->num_merges = 0;
 	bp->curpos     = 0;
 	bp->length     = 0;
 	bp->data       = 0;
 	bp->streamPos  = bp->pos->tell (bp->pos);
 
-	BIFF_SET_GUINT16(data, opcode);
-	BIFF_SET_GUINT16(data+2,0xfaff); /* To be corrected later */
+	MS_OLE_SET_GUINT16(data, opcode);
+	MS_OLE_SET_GUINT16(data+2,0xfaff); /* To be corrected later */
 	bp->pos->write (bp->pos, data, 4);
 }
 void
@@ -417,8 +422,8 @@ ms_biff_put_var_commit (BiffPut *bp)
 	endpos = bp->streamPos + bp->length + 4;
 	bp->pos->lseek (bp->pos, bp->streamPos, MsOleSeekSet);
 
-	BIFF_SET_GUINT16 (tmp, (bp->ms_op<<8) + bp->ls_op);
-	BIFF_SET_GUINT16 (tmp+2, bp->length);
+	MS_OLE_SET_GUINT16 (tmp, (bp->ms_op<<8) + bp->ls_op);
+	MS_OLE_SET_GUINT16 (tmp+2, bp->length);
 	bp->pos->write (bp->pos, tmp, 4);
 
 	bp->pos->lseek (bp->pos, endpos, MsOleSeekSet);
@@ -439,8 +444,8 @@ ms_biff_put_len_commit (BiffPut *bp)
 /*	if (!bp->data_malloced) Unimplemented optimisation
 		bp->pos->lseek (bp->pos, bp->length, MsOleSeekCur);
 		else */
-	BIFF_SET_GUINT16 (tmp, (bp->ms_op<<8) + bp->ls_op);
-	BIFF_SET_GUINT16 (tmp+2, bp->length);
+	MS_OLE_SET_GUINT16 (tmp, (bp->ms_op<<8) + bp->ls_op);
+	MS_OLE_SET_GUINT16 (tmp+2, bp->length);
 	bp->pos->write (bp->pos, tmp, 4);
 	bp->pos->write (bp->pos, bp->data, bp->length);
 
