@@ -8,22 +8,6 @@
 #define GNUMERIC_BIFF_H
 #include "ms-ole.h"
 
-#define BIFF_GET_GUINT8(p) (*((const guint8 *)(p)+0))
-#define BIFF_GET_GUINT16(p) (guint16)(*((const guint8 *)(p)+0) | (*((const guint8 *)(p)+1)<<8))
-#define BIFF_GET_GUINT32(p) (guint32)(*((const guint8 *)(p)+0) | \
-                        (*((const guint8 *)(p)+1)<<8) | \
-                        (*((const guint8 *)(p)+2)<<16) | \
-                        (*((const guint8 *)(p)+3)<<24))
-#define BIFF_GET_GUINT64(p) (BIFF_GET_GUINT32(p) | (((Dguint32)BIFF_GET_GUINT32((const guint8 *)(p)+4))<<32))
-
-#define BIFF_SET_GUINT8(p,n)  (*((guint8 *)(p)+0)=n)
-#define BIFF_SET_GUINT16(p,n) ((*((guint8 *)(p)+0)=((n)&0xff)), \
-                               (*((guint8 *)(p)+1)=((n)>>8)&0xff))
-#define BIFF_SET_GUINT32(p,n) ((*((guint8 *)(p)+0)=((n))&0xff), \
-                               (*((guint8 *)(p)+1)=((n)>>8)&0xff), \
-                               (*((guint8 *)(p)+2)=((n)>>16)&0xff), \
-                               (*((guint8 *)(p)+3)=((n)>>24)&0xff))
-
 extern double biff_getdouble (const guint8 *p);
 extern void   biff_setdouble (guint8 *p, double d);
 	
@@ -42,12 +26,15 @@ extern void   biff_setdouble (guint8 *p, double d);
 /*                                 Read Side                                   */
 /*******************************************************************************/
 
+typedef struct _BiffQuery BiffQuery;
+
+typedef guint16 (BiffQuirkFn) (const BiffQuery *, guint16 op);
+
 /**
  * Returns query data, it is imperative that copies of
  * 'data *' should _not_ be kept.
  **/
-typedef struct _BiffQuery
-{
+struct _BiffQuery {
 	guint8  ms_op;
 	guint8  ls_op;
 	guint16 opcode;
@@ -55,22 +42,26 @@ typedef struct _BiffQuery
 	guint8  *data;
 	guint32 streamPos;
 	guint16 num_merges;
-	gint16  padding;
+/*	gint16  padding;*/
 	int     data_malloced; /* is *data a copy ? */
 	MsOleStream *pos;
-} BiffQuery;
+	BiffQuirkFn *quirk;    /* Ugly ! */
+};
  
 /* Sets up a query on a stream */
-extern BiffQuery   *ms_biff_query_new        (MsOleStream *);
+extern BiffQuery  *ms_biff_query_new         (MsOleStream *);
+/*extern void        ms_biff_query_set_quirk   (BiffQuery *,
+  BiffQuirkFn *);*/
 /* Duplicates this query, so chaining can re-commence here */
-extern BiffQuery   *ms_biff_query_copy       (const BiffQuery *p);
+extern BiffQuery  *ms_biff_query_copy        (const BiffQuery *p);
 /* Updates the BiffQuery structure with the next BIFF record
  * returns: 1 for succes, and 0 for EOS(tream) */
-extern int           ms_biff_query_next_merge (BiffQuery *, gboolean do_merge);
-#define       ms_biff_query_next(q)    ms_biff_query_next_merge ((q), TRUE)
+extern int         ms_biff_query_next_merge (BiffQuery *, gboolean do_merge);
+#define            ms_biff_query_next(q)     ms_biff_query_next_merge ((q), TRUE)
+extern int         ms_biff_query_peek_next  (BiffQuery *, guint16 *opcode);
 /* Converts a merged query to the un-merged equivalent */
-extern void          ms_biff_query_unmerge    (BiffQuery *);
-extern void          ms_biff_query_destroy    (BiffQuery *);
+extern void        ms_biff_query_unmerge    (BiffQuery *);
+extern void        ms_biff_query_destroy    (BiffQuery *);
 
 /*******************************************************************************/
 /*                                 Write Side                                  */
@@ -85,15 +76,17 @@ typedef struct _BiffPut
 	MsOlePos   streamPos;
 	MsOlePos   curpos; /* Curpos is offset from beggining of header */
 	guint16        num_merges;      
-	gint16         padding;
+/*	gint16         padding;*/
 	int            data_malloced;
 	int            len_fixed;
 	MsOleStream *pos;
+	BiffQuirkFn *quirk;    /* Ugly ! */
 } BiffPut;
  
 /* Sets up a record on a stream */
 extern BiffPut      *ms_biff_put_new        (MsOleStream *);
-extern void           ms_biff_put_destroy    (BiffPut *);
+extern void          ms_biff_put_destroy    (BiffPut *);
+
 /**
  * If between the 'next' and 'commit' ls / ms_op are changed they will be
  * written correctly.
