@@ -10,17 +10,18 @@
  **/
 
 #include "excel.h"
+#include "ms-biff.h"
 
 /* Pass this a BiffQuery * */
-#define EX_GETROW(p)      (BIFF_GET_GUINT16(p->data + 0))
-#define EX_GETCOL(p)      (BIFF_GET_GUINT16(p->data + 2))
-#define EX_GETXF(p)       (BIFF_GET_GUINT16(p->data + 4))
-#define EX_GETSTRLEN(p)   (BIFF_GET_GUINT16(p->data + 6))
+#define EX_GETROW(p)      (MS_OLE_GET_GUINT16(p->data + 0))
+#define EX_GETCOL(p)      (MS_OLE_GET_GUINT16(p->data + 2))
+#define EX_GETXF(p)       (MS_OLE_GET_GUINT16(p->data + 4))
+#define EX_GETSTRLEN(p)   (MS_OLE_GET_GUINT16(p->data + 6))
 
-#define EX_SETROW(p,d)    (BIFF_SET_GUINT16(p + 0, d))
-#define EX_SETCOL(p,d)    (BIFF_SET_GUINT16(p + 2, d))
-#define EX_SETXF(p,d)     (BIFF_SET_GUINT16(p + 4, d))
-#define EX_SETSTRLEN(p,d) (BIFF_SET_GUINT16(p + 6, d))
+#define EX_SETROW(p,d)    (MS_OLE_SET_GUINT16(p + 0, d))
+#define EX_SETCOL(p,d)    (MS_OLE_SET_GUINT16(p + 2, d))
+#define EX_SETXF(p,d)     (MS_OLE_SET_GUINT16(p + 4, d))
+#define EX_SETSTRLEN(p,d) (MS_OLE_SET_GUINT16(p + 6, d))
 
 /* Version info types as found in various Biff records */
 typedef enum _eBiff_filetype { eBiffTWorkbook=0, eBiffTVBModule=1, eBiffTWorksheet=2,
