@@ -8,6 +8,8 @@
 
 #include "ms-obj.h"
 #include "ms-chart.h"
+#include "ms-escher.h"
+#include "utils.h"
 
 extern int ms_excel_read_debug;
 
@@ -85,14 +87,14 @@ ms_obj_read_text_impl (BiffQuery *q, ExcelWorkbook * wb)
 	    "At bottom", "Verticaly justified"
 	};
 
-	guint16 const options = BIFF_GET_GUINT16 (q->data);
-	guint16 const orient = BIFF_GET_GUINT16 (q->data+2);
-	guint16 const text_len = BIFF_GET_GUINT16 (q->data+10);
-	guint16 const num_formats = BIFF_GET_GUINT16 (q->data+12);
+	guint16 const options = MS_OLE_GET_GUINT16 (q->data);
+	guint16 const orient = MS_OLE_GET_GUINT16 (q->data+2);
+	guint16 const text_len = MS_OLE_GET_GUINT16 (q->data+10);
+	guint16 const num_formats = MS_OLE_GET_GUINT16 (q->data+12);
 	int const halign = (options >> 1) & 0x7;
 	int const valign = (options >> 4) & 0x7;
 	char * text = g_new(char, text_len+1);
-	guint8 const unicode_flag = BIFF_GET_GUINT8 (q->data+18);
+	guint8 const unicode_flag = MS_OLE_GET_GUINT8 (q->data+18);
 	guchar const * ptr;
 	int i, increment = 1;
 
@@ -134,26 +136,37 @@ ms_obj_read_text_impl (BiffQuery *q, ExcelWorkbook * wb)
 		printf ("}; /* TextObject */\n");
 	}
 }
-static void
+void
 ms_obj_read_text (BiffQuery *q, ExcelWorkbook * wb, int const id)
 {
-	/* next record must be a DRAWING followed by a TXO */
+	/* next record must be a DRAWING */
 	g_return_if_fail (ms_biff_query_next (q));
 	g_return_if_fail (q->opcode == BIFF_MS_O_DRAWING);
+	ms_escher_hack_get_drawing (q, wb);
 
-	if (ms_excel_read_debug > 0)
+	if (ms_excel_read_debug > 1)
 		dump (q->data, q->length);
 
-	/* and finally a TXO */
+	/* then a TXO, CONTINUE, CONTINUE */
 	g_return_if_fail (ms_biff_query_next (q));
 	g_return_if_fail (q->opcode == BIFF_TXO);
 	ms_obj_read_text_impl (q, wb);
+
+#if 0
+	/* FIXME : Most but not all have a trailing Drawing */
+	/* finally another DRAWING */
+	g_return_if_fail (ms_biff_query_next (q));
+	if (q->opcode != BIFF_MS_O_DRAWING)
+		printf ("Expected MS_O_DRAWING found 0x%x\n", q->opcode);
+	else
+		ms_escher_hack_get_drawing (q);
+#endif
 }
 
 static void
 ms_obj_dump (guint8 const * const data, int const len, char const * const name)
 {
-	if (ms_excel_read_debug == 0)
+	if (ms_excel_read_debug < 2)
 		return;
 
 	printf ("{ %s \n", name);
@@ -161,6 +174,52 @@ ms_obj_dump (guint8 const * const data, int const len, char const * const name)
 	printf ("}; /* %s */\n", name);
 }
 
+
+static void
+ms_obj_read_pre_biff8_obj (BiffQuery *q, ExcelWorkbook * wb)
+{
+	static char const * const obj_types[] = {
+	    "Group", "Line", "Rectangle", "Oval", "Arc", "Chart", "Text",
+	    "Button", "Picture", "Polygon", "Checkbox", "OptionButton",
+	    "Edit box", "Label", "Dialog frame", "Spinner", "Listbox",
+	    "Group box", "Dropdown"
+	};
+#if 0
+	guint32 const numObjects = MS_OLE_GET_GUINT32(q->data);
+	guint16 const flags = MS_OLE_GET_GUINT32(q->data+8);
+#endif
+	guint16 tmp = MS_OLE_GET_GUINT32(q->data+4);
+	guint16 const obj_id = MS_OLE_GET_GUINT32(q->data+6);
+
+	guint16 const left_col = MS_OLE_GET_GUINT32(q->data+10);
+	guint16 const top_row = MS_OLE_GET_GUINT32(q->data+14);
+	guint16 const right_col = MS_OLE_GET_GUINT32(q->data+18);
+	guint16 const bottom_row = MS_OLE_GET_GUINT32(q->data+22);
+
+	/* As 1/1024 of cell width */
+	guint16 const left_offset = MS_OLE_GET_GUINT32(q->data+12);
+	guint16 const top_offset = MS_OLE_GET_GUINT32(q->data+16);
+	guint16 const right_offset = MS_OLE_GET_GUINT32(q->data+20);
+	guint16 const bottom_offset = MS_OLE_GET_GUINT32(q->data+24);
+
+	if (tmp >= (sizeof(obj_types)/sizeof(char const * const))) {
+		printf ("EXCEL : invalid object type %d\n", tmp);
+		return;
+	}
+
+	if (ms_excel_read_debug > 0) {
+		printf ("EXCEL : Found %s @ (%s%d + %f %%, %f %%):(%s%d + %f %%, %f %%)\n",
+			obj_types[tmp],
+			col_name(left_col), top_row+1,
+			left_offset/1024., top_offset/1024.,
+			col_name(right_col), bottom_row+1,
+			right_offset/1024., bottom_offset/1024.);
+	}
+
+	if (tmp == 0x5)
+		ms_excel_read_chart (q, wb, obj_id);
+}
+
 static void
 ms_obj_read_biff8_obj (BiffQuery *q, ExcelWorkbook * wb)
 {
@@ -178,10 +237,10 @@ ms_obj_read_biff8_obj (BiffQuery *q, ExcelWorkbook * wb)
 
 	/* Scan through the pseudo BIFF substream */
 	while (data_len_left > 0 && !hit_end) {
-		guint16 const record_type = BIFF_GET_GUINT16(data);
+		guint16 const record_type = MS_OLE_GET_GUINT16(data);
 
 		/* All the sub-records seem to have this layout */
-		guint16 const len = BIFF_GET_GUINT16(data+2);
+		guint16 const len = MS_OLE_GET_GUINT16(data+2);
 
 		/* 1st record must be COMMON_OBJ*/
 		g_return_if_fail (obj_type >= 0 ||
@@ -194,83 +253,89 @@ ms_obj_read_biff8_obj (BiffQuery *q, ExcelWorkbook * wb)
 			break;
 
 		case GR_MACRO :
-			ms_obj_dump (data+4, len, "MacroObject");
+			ms_obj_dump (data, len, "MacroObject");
 			break;
 
 		case GR_COMMAND_BUTTON :
-			ms_obj_dump (data+4, len, "CommandButton");
+			ms_obj_dump (data, len, "CommandButton");
 			break;
 
 		case GR_GROUP_BUTTON :
-			ms_obj_dump (data+4, len, "GroupButton");
+			ms_obj_dump (data, len, "GroupButton");
 			break;
 
 		case GR_CLIPBOARD_FORMAT:
-			ms_obj_dump (data+4, len, "ClipboardFmt");
+			ms_obj_dump (data, len, "ClipboardFmt");
 			break;
 
 		case GR_PICTURE_OPTIONS:
-			ms_obj_dump (data+4, len, "PictOpt");
+			ms_obj_dump (data, len, "PictOpt");
 			break;
 
 		case GR_PICTURE_FORMULA:
-			ms_obj_dump (data+4, len, "PictFormula");
+			ms_obj_dump (data, len, "PictFormula");
 			break;
 
 		case GR_CHECKBOX_LINK :
-			ms_obj_dump (data+4, len, "CheckboxLink");
+			ms_obj_dump (data, len, "CheckboxLink");
 			break;
 
 		case GR_RADIO_BUTTON :
-			ms_obj_dump (data+4, len, "RadioButton");
+			ms_obj_dump (data, len, "RadioButton");
 			break;
 
 		case GR_SCROLLBAR :
-			ms_obj_dump (data+4, len, "ScrollBar");
+			ms_obj_dump (data, len, "ScrollBar");
 			break;
 
 		case GR_NOTE_STRUCTURE :
-			ms_obj_dump (data+4, len, "Note");
+			ms_obj_dump (data, len, "Note");
 			break;
 
 		case GR_SCROLLBAR_FORMULA :
-			ms_obj_dump (data+4, len, "ScrollbarFmla");
+			ms_obj_dump (data, len, "ScrollbarFmla");
 			break;
 
 		case GR_GROUP_BOX_DATA :
-			ms_obj_dump (data+4, len, "GroupBoxData");
+			ms_obj_dump (data, len, "GroupBoxData");
 			break;
 
 		case GR_EDIT_CONTROL_DATA :
-			ms_obj_dump (data+4, len, "EditCtrlData");
+			ms_obj_dump (data, len, "EditCtrlData");
 			break;
 
 		case GR_RADIO_BUTTON_DATA :
-			ms_obj_dump (data+4, len, "RadioData");
+			ms_obj_dump (data, len, "RadioData");
 			break;
 
 		case GR_CHECKBOX_DATA :
-			ms_obj_dump (data+4, len, "CheckBoxData");
+			ms_obj_dump (data, len, "CheckBoxData");
 			break;
 
 		case GR_LISTBOX_DATA :
-			ms_obj_dump (data+4, len, "ListBoxData");
+			ms_obj_dump (data, len, "ListBoxData");
 			break;
 
 		case GR_CHECKBOX_FORMULA :
-			ms_obj_dump (data+4, len, "CheckBoxFmla");
+		{
+			guint16 const row = MS_OLE_GET_GUINT16(data+11);
+			guint16 const col = MS_OLE_GET_GUINT16(data+13) &0x3fff;
+			if (ms_excel_read_debug > 0)
+				printf ("Checkbox linked to : %s%d\n", col_name(col), row+1);
+			ms_obj_dump (data, len, "CheckBoxFmla");
 			break;
+		}
 
 		case GR_COMMON_OBJ_DATA:
 		{
 			char const *type_name = NULL;
-			guint16 const options =BIFF_GET_GUINT16(data+8);
+			guint16 const options =MS_OLE_GET_GUINT16(data+8);
 
 			/* Multiple objects in 1 record ?? */
 			g_return_if_fail (obj_type == -1);
 
-			obj_type = BIFF_GET_GUINT16(data+4);
-			obj_id = BIFF_GET_GUINT16(data+6);
+			obj_type = MS_OLE_GET_GUINT16(data+4);
+			obj_id = MS_OLE_GET_GUINT16(data+6);
 			if (obj_type<sizeof(object_type_names)/sizeof(char*))
 				type_name =object_type_names[obj_type];
 			else
@@ -311,8 +376,15 @@ ms_obj_read_biff8_obj (BiffQuery *q, ExcelWorkbook * wb)
 	/* If this was a Chart then there should be a BOF next */
 	switch (obj_type)
 	{
-	case 0x05 : ms_excel_read_chart (q, wb, obj_id); break;
-	case 0x06 : ms_obj_read_text (q, wb, obj_id); break;
+	case 0x05 :
+		ms_excel_read_chart (q, wb, obj_id);
+		break;
+
+	case 0x06 : /* Text Box */
+	case 0x07 : /* Button */
+		ms_obj_read_text (q, wb, obj_id);
+		break;
+
 	default:
 		    break;
 	}
@@ -327,5 +399,5 @@ ms_obj_read_obj (BiffQuery *q, ExcelWorkbook * wb)
 	if (wb->ver >= eBiffV8)
 		ms_obj_read_biff8_obj (q, wb);
 	else
-		printf ("Only Biff8 Objects are supporting currently\n");
+		ms_obj_read_pre_biff8_obj (q, wb);
 }
