@@ -12,7 +12,7 @@
 #define GNUMERIC_SHEET_CLASS(k) (GTK_CHECK_CLASS_CAST ((k), GNUMERIC_TYPE_SHEET))
 #define GNUMERIC_IS_SHEET(o)    (GTK_CHECK_TYPE((o), GNUMERIC_TYPE_SHEET))
 
-#define GNUMERIC_SHEET_PATTERNS 13
+#define GNUMERIC_SHEET_PATTERNS 18
 typedef struct {
 	char *name;
 	char pattern [8];
@@ -26,8 +26,8 @@ typedef struct {
 	GtkWidget     *entry;
 	SheetView     *sheet_view;
 	
-	ColType       top_col, last_visible_col, last_full_col;
-	RowType       top_row, last_visible_row, last_full_row;
+	int           top_col, last_visible_col, last_full_col;
+	int           top_row, last_visible_row, last_full_row;
 
 	ItemGrid      *item_grid;
 	ItemCursor    *item_cursor;
