@@ -295,27 +295,27 @@ item_grid_draw_cell (GdkDrawable *drawable, ItemGrid *item_grid, Cell *cell, int
 		GdkGC           *gc;
 		StyleBorderType  t;
 
-		t = b->type[STYLE_TOP];
+		t = b->type [STYLE_TOP];
 		if (t != BORDER_NONE) {
-			gc = item_grid->border_gc[b->type[STYLE_TOP]];
+			gc = item_grid->border_gc [t];
 			gdk_draw_line (drawable, gc, x1, y1, x1 + w, y1);
 		}
 
-		t = b->type[STYLE_BOTTOM];
+		t = b->type [STYLE_BOTTOM];
 		if (t != BORDER_NONE) {
-			gc = item_grid->border_gc[b->type[STYLE_BOTTOM]];
+			gc = item_grid->border_gc [t];
 			gdk_draw_line (drawable, gc, x1, y1 + h, x1 + w, y1 + h);
 		}
 
-		t = b->type[STYLE_LEFT];
+		t = b->type [STYLE_LEFT];
 		if (t != BORDER_NONE) {
-			gc = item_grid->border_gc[b->type[STYLE_LEFT]];
+			gc = item_grid->border_gc [t];
 			gdk_draw_line (drawable, gc, x1, y1, x1, y1 + h);
 		}
 
-		t = b->type[STYLE_RIGHT];
+		t = b->type [STYLE_RIGHT];
 		if (t != BORDER_NONE) {
-			gc = item_grid->border_gc[b->type[STYLE_RIGHT]];
+			gc = item_grid->border_gc [t];
 			gdk_draw_line (drawable, gc, x1 + w, y1, x1 + w, y1 + h);
 		}
 	}
@@ -325,6 +325,34 @@ item_grid_draw_cell (GdkDrawable *drawable, ItemGrid *item_grid, Cell *cell, int
 	return count;
 }
 
+static void
+item_grid_paint_empty_cell (GdkDrawable *drawable, ItemGrid *item_grid,
+			    ColRowInfo *ci, ColRowInfo *ri, int col, int row,
+			    int x, int y)
+{
+	Style *style;
+	
+	style = sheet_style_compute (item_grid->sheet, col, row, NULL);
+
+	if (style->valid_flags & (STYLE_PATTERN | STYLE_FORE_COLOR)){
+		/* FIXME: set the GC here */
+	}
+
+	if ((style->valid_flags & STYLE_BACK_COLOR) && (style->back_color)){
+
+		/*
+		 * FIXME: If the style is the default sheet style, we can
+		 * avoid drawing this
+		 */
+		gdk_gc_set_foreground (item_grid->gc, &style->back_color->color);
+		gdk_draw_rectangle (
+			drawable, item_grid->gc, TRUE,
+			x + ci->margin_a, y + ri->margin_b,
+			ci->pixels - ci->margin_b,
+			ri->pixels - ri->margin_b);
+	}
+}
+
 static void
 item_grid_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int width, int height)
 {
@@ -403,9 +431,18 @@ item_grid_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int
 			cell = sheet_cell_get (sheet, col, row);
 
 			if (cell){
-				item_grid_draw_cell (drawable, item_grid, cell,
-						     x_paint, y_paint);
-			} else if (ri->pos != -1){
+				item_grid_draw_cell (
+					drawable, item_grid, cell,
+					x_paint, y_paint);
+			}
+
+			if (!cell && gnumeric_debugging > 0){
+				item_grid_paint_empty_cell (
+					drawable, item_grid, ci, ri,
+					col, row, x_paint, y_paint);
+			}
+			
+			if (!cell && (ri->pos != -1)){
 				/*
 				 * If there was no cell, and the row has any cell allocated
 				 * (indicated by ri->pos != -1)
@@ -704,7 +741,8 @@ item_grid_button_1 (Sheet *sheet, GdkEvent *event, ItemGrid *item_grid, int col,
 
 	if (!(event->button.state & GDK_SHIFT_MASK)) {
 		sheet_make_cell_visible (sheet, col, row);
-		sheet_cursor_move (sheet, col, row);
+		/* Handle the selection localy */
+		sheet_cursor_move (sheet, col, row, FALSE, FALSE);
 	}
 
 	if (!(event->button.state & (GDK_CONTROL_MASK|GDK_SHIFT_MASK)))
