@@ -258,8 +258,13 @@ sheet_compute_col_row_new_size (Sheet *sheet, ColRowInfo *ci, void *data)
 	double pix_per_unit = sheet->last_zoom_factor_used;
 
 	ci->pixels = (ci->units + ci->margin_a_pt + ci->margin_b_pt) * pix_per_unit;
-	ci->margin_a = ci->margin_a_pt * pix_per_unit;
-	ci->margin_b = ci->margin_b_pt * pix_per_unit;
+
+	/*
+	 * Ensure that there is at least 1 pixel around every cell so that we
+	 * can mark the current cell
+	 */
+	ci->margin_a = MAX(ci->margin_a_pt * pix_per_unit, 1);
+	ci->margin_b = MAX(ci->margin_b_pt * pix_per_unit, 1);
 }
 
 static Value *
@@ -288,7 +293,7 @@ sheet_set_zoom_factor (Sheet *sheet, double factor)
 
 	/* First, the default styles */
 	sheet_compute_col_row_new_size (sheet, &sheet->default_row_style, NULL);
- 	sheet_compute_col_row_new_size (sheet, &sheet->default_col_style, NULL);
+	sheet_compute_col_row_new_size (sheet, &sheet->default_col_style, NULL);
 
 	/* Then every column and row */
 	sheet_foreach_col (sheet, sheet_compute_col_row_new_size, NULL);
@@ -511,9 +516,9 @@ sheet_row_info_set_height (Sheet *sheet, ColRowInfo *ri, int height, gboolean he
 
 /**
  * sheet_row_set_height:
- * @sheet:         	The sheet
- * @row:           	The row
- * @height:        	The desired height
+ * @sheet:		The sheet
+ * @row:		The row
+ * @height:		The desired height
  * @height_set_by_user: TRUE if this was done by a user (ie, user manually
  *                      set the width)
  *
@@ -544,9 +549,9 @@ sheet_row_set_height (Sheet *sheet, int row, int height, gboolean height_set_by_
 
 /**
  * sheet_row_set_internal_height:
- * @sheet:         	The sheet
- * @row:           	The row
- * @height:        	The desired height
+ * @sheet:		The sheet
+ * @row:		The row
+ * @height:		The desired height
  *
  * Sets the height of a row in terms of the internal required space (the total
  * size of the row will include the margins.
@@ -573,6 +578,51 @@ sheet_row_set_internal_height (Sheet *sheet, ColRowInfo *ri, double height)
 	sheet_redraw_all (sheet);
 }
 
+/**
+ * sheet_col_size_fit:
+ * @sheet: The sheet
+ * @col: the column that we want to query
+ *
+ * This routine computes the ideal size for the column to make all data fit
+ * properly.  Return value is in pixels
+ */
+int
+sheet_col_size_fit (Sheet *sheet, int col)
+{
+	ColRowInfo *ci;
+	GList *l;
+	int max = 0;
+	int margins;
+	
+	g_return_val_if_fail (sheet != NULL, 0);
+	g_return_val_if_fail (IS_SHEET (sheet), 0);
+	g_return_val_if_fail (col >= 0, 0);
+	g_return_val_if_fail (col < SHEET_MAX_COLS, 0);
+
+	ci = sheet_col_get_info (sheet, col);
+
+	/*
+	 * If ci == sheet->default_col_style then it means
+	 * no cells have been allocated here
+	 */
+	if (ci == &sheet->default_col_style)
+		return ci->pixels;
+
+	margins = ci->margin_a + ci->margin_b;
+	
+	for (l = ci->data; l; l = l->next){
+		Cell *cell = l->data;
+		int width;
+			
+		width = cell->width + margins;
+		
+		if (width > max)
+			max = width;
+	}
+
+	return max;
+}
+
 /**
  * sheet_recompute_spans_for_col:
  * @sheet: the sheet
@@ -1287,7 +1337,7 @@ sheet_select_all (Sheet *sheet)
 
 	sheet_selection_reset_only (sheet);
 	sheet_make_cell_visible (sheet, 0, 0);
-	sheet_cursor_move (sheet, 0, 0);
+	sheet_cursor_move (sheet, 0, 0, FALSE, FALSE);
 	sheet_selection_append_range (sheet, 0, 0, 0, 0,
 		SHEET_MAX_COLS-1, SHEET_MAX_ROWS-1);
 
@@ -2314,10 +2364,10 @@ sheet_cell_foreach_range (Sheet *sheet, int only_existing,
 static Value *
 fail_if_not_selected (Sheet *sheet, int col, int row, Cell *cell, void *user_data)
 {
- 	if (!sheet_selection_is_cell_selected (sheet, col, row))
-  		return value_terminate();
- 	else
- 		return NULL;
+	if (!sheet_selection_is_cell_selected (sheet, col, row))
+		return value_terminate();
+	else
+		return NULL;
 }
 
 /**
@@ -3073,7 +3123,6 @@ static void
 sheet_move_column (Sheet *sheet, ColRowInfo *ci, int new_column)
 {
 	GList *rows, *column_cells, *l;
-	int diff = new_column - ci->pos;
 
 	/* remove the cells */
 	column_cells = NULL;
@@ -3093,7 +3142,7 @@ sheet_move_column (Sheet *sheet, ColRowInfo *ci, int new_column)
 
 		sheet_cell_add_to_hash (sheet, cell);
 
-		cell_relocate (cell, diff, 0);
+		cell_relocate (cell, 0, 0);
 	}
 	g_list_free (column_cells);
 }
@@ -3362,7 +3411,7 @@ sheet_shift_row (Sheet *sheet, int col, int row, int count)
 		/* Relocate the cell */
 		sheet_cell_remove (sheet, cell);
 		sheet_cell_add (sheet, cell, new_column, row);
-		cell_relocate (cell, count, 0);
+		cell_relocate (cell, 0, 0);
 	}
 	g_list_free (l2);
 
@@ -3497,7 +3546,7 @@ sheet_insert_row (Sheet *sheet, int row, int count)
 
 		sheet_cell_add_to_hash (sheet, cell);
 
-		cell_relocate (cell, 0, count);
+		cell_relocate (cell, 0, 0);
 	}
 
 	g_list_free (cell_store);
@@ -3603,7 +3652,7 @@ sheet_delete_row (Sheet *sheet, int row, int count)
 
 		sheet_cell_add_to_hash (sheet, cell);
 
-		cell_relocate (cell, 0, -count);
+		cell_relocate (cell, 0, 0);
 	}
 	g_list_free (cell_store);
 
@@ -3700,7 +3749,7 @@ sheet_shift_col (Sheet *sheet, int col, int row, int count)
 		}
 
 		sheet_cell_add (sheet, cell, col, new_row);
-		cell_relocate (cell, 0, count);
+		cell_relocate (cell, 0, 0);
 	}
 	g_list_free (cell_list);
 
@@ -3758,9 +3807,9 @@ sheet_style_attach (Sheet *sheet, int start_col, int start_row, int end_col, int
 
 /**
  * sheet_style_compute:
- * @sheet:   	 Which sheet we are looking up
- * @col:     	 column
- * @row:     	 row
+ * @sheet:	 Which sheet we are looking up
+ * @col:	 column
+ * @row:	 row
  * @non_default: A pointer where we store the attributes
  *               the cell has which are not part of the
  *               default style.
@@ -3816,16 +3865,34 @@ sheet_make_cell_visible (Sheet *sheet, int col, int row)
 	}
 }
 
+/**
+ * sheet_cursor_move:
+ * @sheet: Which sheet's cursor to move
+ * @col:   destination column
+ * @row:   destination row
+ * @clear_selection:       Clear the old selection if we move.
+ * @add_dest_to_selection: Add the new cursor location to the
+ *                         selection if we move.
+ *
+ * Adjusts the cursor location for the specified sheet, optionaly
+ * clearing the old selection and adding the new cursor to the selection.
+ * Be careful when manging the selection when the 'move' returns to the
+ * current location.
+ */
 void
-sheet_cursor_move (Sheet *sheet, int col, int row)
+sheet_cursor_move (Sheet *sheet, int col, int row,
+		   gboolean clear_selection, gboolean add_dest_to_selection)
 {
 	GList *l;
+	int old_row, old_col;
 
 	g_return_if_fail (sheet != NULL);
 	g_return_if_fail (IS_SHEET (sheet));
 
 	sheet_accept_pending_input (sheet);
 
+	old_row = sheet->cursor_col;
+	old_col = sheet->cursor_row;
 	sheet->cursor_col = col;
 	sheet->cursor_row = row;
 
@@ -3836,6 +3903,13 @@ sheet_cursor_move (Sheet *sheet, int col, int row)
 		gnumeric_sheet_set_cursor_bounds (gsheet, col, row, col, row);
 	}
 	sheet_load_cell_val (sheet);
+
+	if (old_row != row || old_col != col) {
+		if (clear_selection)
+			sheet_selection_reset_only (sheet);
+		if (add_dest_to_selection)
+			sheet_selection_append (sheet, col, row);
+	}
 }
 
 void
@@ -3867,8 +3941,8 @@ sheet_cursor_set (Sheet *sheet, int base_col, int base_row, int start_col, int s
 
 /**
  * sheet_fill_selection_with:
- * @sheet:   	 Which sheet we are operating on.
- * @str:     	 The text to fill the selection with.
+ * @sheet:	 Which sheet we are operating on.
+ * @str:	 The text to fill the selection with.
  * @is_array:    A flag to differentiate between filling and array formulas.
  *
  * Checks to ensure that none of the ranges being filled contain a subset of
