@@ -22,7 +22,7 @@
 #include "pixmaps.h"
 #include "clipboard.h"
 #include "utils.h"
-#include "widget-editable-label.h"
+#include "widgets/widget-editable-label.h"
 #include "print-info.h"
 #include "ranges.h"
 
@@ -45,15 +45,7 @@ static int workbook_count;
 
 static GList *workbook_list = NULL;
 
-/*
- * For the Workbook class
- */
-enum {
-	CELL_CONTENT_CHANGED,
-	LAST_SIGNAL,
-};
 static WORKBOOK_PARENT_CLASS *workbook_parent_class;
-static guint wb_signals [LAST_SIGNAL] = { 0, };
 
 static void workbook_set_focus (GtkWindow *window, GtkWidget *focus, Workbook *wb);
 
@@ -105,7 +97,7 @@ file_save_as_cmd (GtkWidget *widget, Workbook *wb)
 static void
 summary_cmd (GtkWidget *widget, Workbook *wb)
 {
-	dialog_summary_update (widget, wb->sin);
+	dialog_summary_update (widget, wb->summary_info);
 }
 
 static void
@@ -188,18 +180,14 @@ change_selection_font (Workbook *wb, int bold, int italic)
 }
 
 static void
-bold_cmd (GtkWidget *widget, Workbook *wb)
+bold_cmd (GtkToggleButton *t, Workbook *wb)
 {
-	GtkToggleButton *t = GTK_TOGGLE_BUTTON (widget);
-
 	change_selection_font (wb, t->active, -1);
 }
 
 static void
-italic_cmd (GtkWidget *widget, Workbook *wb)
+italic_cmd (GtkToggleButton *t, Workbook *wb)
 {
-	GtkToggleButton *t = GTK_TOGGLE_BUTTON (widget);
-
 	change_selection_font (wb, -1, t->active);
 }
 
@@ -214,6 +202,15 @@ create_graphic_cmd (GtkWidget *widget, Workbook *wb)
 }
 #endif
 
+static void
+create_button_cmd (GtkWidget *widget, Workbook *wb)
+{
+	Sheet *sheet;
+
+	sheet = workbook_get_current_sheet (wb);
+	sheet_set_mode_type (sheet, SHEET_MODE_CREATE_BUTTON);
+}
+
 static void
 create_line_cmd (GtkWidget *widget, Workbook *wb)
 {
@@ -280,8 +277,8 @@ workbook_do_destroy (Workbook *wb)
 	 * order.
 	 */
 
-	summary_info_free (wb->sin);
-	wb->sin = NULL;
+	summary_info_free (wb->summary_info);
+	wb->summary_info = NULL;
 
 	/*
 	 * Erase all cells.  In particular this removes all links between
@@ -858,7 +855,13 @@ print_setup_cmd (GtkWidget *widget, Workbook *wb)
 static void
 file_print_cmd (GtkWidget *widget, Workbook *wb)
 {
-	workbook_print (wb);
+	workbook_print (wb, FALSE);
+}
+
+static void
+file_print_preview_cmd (GtkWidget *widget, Workbook *wb)
+{
+	workbook_print (wb, TRUE);
 }
 
 static void
@@ -929,7 +932,9 @@ static GnomeUIInfo workbook_menu_file [] = {
 
 	GNOMEUIINFO_MENU_PRINT_SETUP_ITEM(print_setup_cmd, NULL),
 	GNOMEUIINFO_MENU_PRINT_ITEM(file_print_cmd, NULL),
-
+	{ GNOME_APP_UI_ITEM, N_("Print pre_view"), N_("Print preview"),
+	  file_print_preview_cmd },
+	
 	GNOMEUIINFO_SEPARATOR,
 
 	GNOMEUIINFO_MENU_CLOSE_ITEM(close_cmd, NULL),
@@ -1005,7 +1010,7 @@ static GnomeUIInfo workbook_menu_view [] = {
 
 static GnomeUIInfo workbook_menu_insert_special [] = {
 	{ GNOME_APP_UI_ITEM, N_("Current _date"),
-	  N_("Insert the current data into the selected cell(s)"),
+	  N_("Insert the current date into the selected cell(s)"),
 	  insert_current_date_cmd,
 	  NULL, NULL, 0, 0, ';', GDK_CONTROL_MASK },
 	{ GNOME_APP_UI_ITEM, N_("Current _time"),
@@ -1155,6 +1160,9 @@ static GnomeUIInfo workbook_toolbar [] = {
 		N_("Graphic"), N_("Creates a graphic in the spreadsheet"),
 		create_graphic_cmd, NULL, graphic_xpm),
 #endif
+	GNOMEUIINFO_ITEM_DATA (
+		N_("Button"), N_("Creates a button object"),
+		create_button_cmd, NULL, button_xpm),
 	GNOMEUIINFO_ITEM_DATA (
 		N_("Line"), N_("Creates a line object"),
 		create_line_cmd, NULL, line_xpm),
@@ -1404,7 +1412,7 @@ workbook_parse_and_jump (Workbook *wb, const char *text)
 #endif
 
 		sheet_make_cell_visible (sheet, col, row);
-		sheet_cursor_move (sheet, col, row);
+		sheet_cursor_move (sheet, col, row, TRUE, TRUE);
 		return TRUE;
 	}
 }
@@ -1484,7 +1492,7 @@ deps_output (GtkWidget *widget, Workbook *wb)
 	Cell *cell;
 	GList *list;
 
-	summary_info_dump (wb->sin);
+	summary_info_dump (wb->summary_info);
 
 	if (!sheet_selection_first_range (
 		sheet, &dummy, &dummy, &col, &row, &dummy, &dummy)){
@@ -1922,8 +1930,8 @@ workbook_init (GtkObject *object)
 	wb->print_info   = print_info_new ();
 	wb->symbol_names = symbol_table_new ();
 	wb->max_iterations = 1;
-	wb->sin          = summary_info_new ();
-	summary_info_default (wb->sin);
+	wb->summary_info   = summary_info_new ();
+	summary_info_default (wb->summary_info);
 
 	/* Set the default operation to be performed over selections */
 	wb->auto_expr      = NULL;
@@ -1948,19 +1956,6 @@ workbook_class_init (GtkObjectClass *object_class)
 	
 	workbook_parent_class = gtk_type_class (WORKBOOK_PARENT_CLASS_TYPE);
 	object_class->destroy = workbook_destroy;
-
-	wb_signals [CELL_CONTENT_CHANGED] =
-		gtk_signal_new (
-			"cell_content_changed",
-			GTK_RUN_LAST,
-			object_class->type,
-			GTK_SIGNAL_OFFSET (WorkbookClass, cell_content_changed),
-			gtk_marshal_NONE__POINTER,
-			GTK_TYPE_NONE, 0);
-
-	workbook_class->cell_content_changed = NULL;
-	
-	gtk_object_class_add_signals (object_class, wb_signals, LAST_SIGNAL);
 }
 
 GtkType
@@ -2218,7 +2213,7 @@ sheet_action_delete_sheet (GtkWidget *widget, Sheet *current_sheet)
 		return;
 
 	message = g_strdup_printf (
-		_("Are you sure you want to remove the sheet called `%s' "),
+		_("Are you sure you want to remove the sheet called `%s'?"),
 		current_sheet->name);
 
 	d = gnome_message_box_new (
@@ -2238,7 +2233,7 @@ sheet_action_delete_sheet (GtkWidget *widget, Sheet *current_sheet)
 		gnumeric_notice (
 			wb, GNOME_MESSAGE_BOX_ERROR,
 			_("Other sheets depend on the values on this sheet; "
-			  "I cannot remove it"));
+			  "I cannot remove it."));
 		return;
 	}
 
@@ -2849,10 +2844,3 @@ workbook_selection_to_string (Workbook *wb, Sheet *base_sheet)
 	return result;
 }
 
-void
-workbook_cell_changed (Workbook *wb, void *c)
-{
-	Cell *cell = c;
-
-	gtk_signal_emit (GTK_OBJECT (wb), wb_signals [CELL_CONTENT_CHANGED], cell);
-}
