@@ -93,7 +93,7 @@ struct _Workbook {
 	SymbolTable *symbol_names;
 
 	/* Attached summary information */
-	SummaryInfo *sin;
+	SummaryInfo *summary_info;
 
 	/*
 	 * This is  used during the clipboard paste command to pass information
@@ -114,10 +114,6 @@ typedef struct {
 #else
 	GtkObjectClass   gtk_parent_class;
 #endif
-	/*
-	 * Signals
-	 */
-	void      (*cell_content_changed) (Workbook *workbook, void *cell);
 } WorkbookClass;
 
 GtkType     workbook_get_type            (void);
@@ -131,7 +127,7 @@ Workbook   *workbook_read                (const char *filename);
 
 void        workbook_save_as             (Workbook *);
 void        workbook_save                (Workbook *);
-void        workbook_print               (Workbook *);
+void        workbook_print               (Workbook *, gboolean);
 void        workbook_attach_sheet        (Workbook *, Sheet *);
 gboolean    workbook_detach_sheet        (Workbook *, Sheet *, gboolean);
 Sheet      *workbook_focus_current_sheet (Workbook *wb);
@@ -198,9 +194,4 @@ extern   Workbook *current_workbook;
 void workbook_corba_setup    (Workbook *);
 void workbook_corba_shutdown (Workbook *);
 
-/*
- * For signaling changes (output synthesis)
- */
-void workbook_cell_changed   (Workbook *wb, void *cell);
-	
 #endif
