@@ -3,7 +3,7 @@
  *
  * Daniel Veillard <Daniel.Veillard@w3.org>
  *
- * $Id: xml-io.h,v 1.7 1999/06/26 19:19:37 unammx Exp $
+ * $Id: xml-io.h,v 1.8 1999/08/11 07:30:36 unammx Exp $
  */
 
 #ifndef GNUMERIC_XML_IO_H
@@ -13,8 +13,8 @@
 
 Sheet    *gnumeric_xml_read_sheet     (const char *filename);
 int       gnumeric_xml_write_sheet    (Sheet *sheet, const char *filename);
-Workbook *gnumeric_xml_read_workbook  (const char *filename);
-int       gnumeric_xml_write_workbook (Workbook *sheet, const char *filename);
+gboolean  gnumeric_xml_read_workbook  (Workbook *wb, const char *filename);
+int       gnumeric_xml_write_workbook (Workbook *wb, const char *filename);
 
 void      xml_init (void);
 
